# shellcheck shell=bash

function exam_env_msg_setup() {
    local _level="${1-1}"

    (( _level >= 0 )) || _level=0

    bb_msg_clear_targets
    case "$_level" in
        0)
            # quiet
            bb_msg_set_target log_stderr stderr err prog notrace
            ;;
        1)
            # normal
            bb_msg_set_target log_stderr stderr info prog notrace
            ;;
        2)
            # verbose
            bb_msg_set_target log_stderr stderr err progpid notrace
            ;;
        3)
            # debug
            bb_msg_set_target log_stderr1 stderr err full trace
            bb_msg_set_target log_stderr2 stderr debug3:warning full notrace
            ;;
        4)
            # debug: errors with stacktraces
            bb_msg_set_target log_stderr1 stderr err full trace
            bb_msg_set_target log_stderr2 notice debug3:warning full notrace
            ;;
        5)
            # debug: all msgs with stracktraces
            bb_msg_set_target log_stderr stderr debug3 full trace
            ;;

    esac
}

function _exam_env_read_config() {
    local _path="$1"

    if [ -r "$_path" ]
    then
        # shellcheck disable=SC1090
        source "$_path" || \
            bb_fatal "read_config: failed to source config from: $_path"
    else
        bb_fatal "read_config: could not find/read config at: $_path"
    fi
}

function _exam_env_server_common_bootstrap() {
    # default message settings
    exam_env_msg_setup 1

    # read config file
    _exam_env_read_config "/etc/default/exam-env-server"
}

_exam_env_server_common_bootstrap
