<!-- -*- mode: markdown; mode: flyspell -*- -->

# LVATEST-TESTENV CONFIG - COURSE STAFF HOWTO 

A lvatest-testenv consists of a dedicated linux installation for each
course deployed on all needed workstations and later modified by a set
of scripts to be switched into lvatest-mode. As staff member you can
do lots of things after the base install was done. It is ideal to
minimize changes so things get not to complicated. If you need
packages or bigger config changes talk the someone of the ZKK staff to
consider integration into the main linux rollout instead.

## naming

The lvatest name is a rather important property since it separates
lots of config and volumes from other courses. The name is referenced
but not set in config.sh (see below). To get a new name means you have
to approach some ZKK staff member to create it and its corresponding
habitat.

The test-name is usually a short string representing the course with
"test" as suffix. This is done so on workstations volumes can easily
be identified as part of a lvatest env.

The test-short-name is the test-name without the "test" suffix. It is
used where short strings are needed and it is obvious that it is used
in the context of some lvatest env.

## apps0

In each lab there exists an application server called
apps0.<labname>.tuwien.ac.at. If your exam runs in multiple labs at
the same time please do not forget to synchronize your config and
priv to all apps0 servers. See below for tools to help you with that.

### access rights & data

Since you read this file you probably always have access
rights. Everyone who is part of the lvatest group (in the non test
environment) can ssh to apps0. You will need this rights in all labs
that you want to use. Make sure you have it, if not please contact the
labs admins.

Each course has its own dir in /home/data/<test-name> which you can
only access by being part of another group: lvatest-<test-short-name>

Important is that all files placed there are readable by that
lvatest-<test-short-name> group, else the workstation will not be able
to access them.

Each course directory has two sub directories: config and priv. The
priv subdir contains lvastaff private files like all the tasks (like
the old .ang dir). It will be synced by the workstation to
/home/lvatestpriv. In hook scripts this path is available in
$LOCAL_PRIV. You can store what ever you want inside. For performance
reasons better do not use archives. Small changes will be much faster,
since rsync is used. Files in $LOCAL_PRIV belong to root:lvatestpriv
and all admin users are member of group lvatestpriv.

If you want to make some of the priv contents to participants, you can
move then out of the priv dir into a location of your choice
(e.g. /home/lvatest/tasks). Move the code that does that into your
bootime-post.sh hook script, which is run after syncing the priv
directory. See below for details.

The config subdir contains all meta information about the test env. It
is described below.

### rollout delay

It is possible to delay the rollout of a lvatest. In fact we strongly
advise to do so, since it will make things much easier: It is possible
to rollout a lvatest-env up to the stage where the lvatest-config is
applied. The remaining rollout steps will only take a short time.

To activate a delay a file named DELAY-ROLLOUT can be created in the
courses directory (not inside the config dir). Empty lines and lines
staring with a hash (#) are ignored. If it contains a line `DELAY`
then the delay is active.

To allow testing of selected workstations there can be another file
called "DELAY-EXCEPTIONS" which may contain a list of hostnames
(without domain) which should ignore any active delay.

There are only two things which will be done even if a rollout is
delayed: First the reboot user will be created and second admin users
will be created. When rollout is continued (as soon as the delay is
either disabled or there is an exception for the checking host) those
users will be updated regarding pwhash and gecos. Any missing users
will be created, too. That allows staff to reboot workstations even if
rollout is delayed.

Once a workstation discovers that a rollout is delayed it will not try
again, but wait for someone to reboot it. Coming up again, it will
check again if rollout can be continued.

## lvatest-config

Each course using the lvatest env should get a base structure as it is
not meant to create it from scratch. If you don't have some please ask
the next friendly ZKK admin.

The config is a configuration blueprint for lvatest client
workstations. While it is stored on apps0 it is not used there. It is
synced every time a workstation boots. Still, it is not fully applied
again and again. Things that happen during rollout time will require a
rerollout of the host, when changed on apps0.

On the client workstation you will a log file where success or
failures of your scripts next to general log messages regarding the
lvatest-env can be found: `/var/log/lvatest-env.log`

The current state of a workstation can be found in `/lvatest-state`.

### bin

Can contain any binary blobs used in some scripts.

### img

Contains images. E.g. if a image called `lightdm-background.png` is
placed inside, lightdm will use it as desktop background (if lightdm
is used).

If no image is installed a default image showing the testname is
created.

When delay is active a watermark showing a DELAY message is
automatically applied.

### netsec

Contains firewall configuration. Be careful in here, if you mess it
up, workstations might be unusable and/or unreachable. Here follows a
short description. Empty lines and lines starting with hash (#) are
ignored.

#### macros

Can be used to define variables in shell script style.

#### input

Contains rules applied to incoming packets. Variables can be used by
prefixing their name by a dollar sign ($). Each line contains one
rule, syntax is similar to iptables(8) arguments.

#### output

Works just like the input file but rules are applied to outgoing
network packets.

Just like the input file but rules are applied to forwarded network
packets. That means packets that are routed by the workstation, which
means this file will most probably stay empty.

#### policies

Contains default rules for default chains. In lvatest env default
rules are to DROP everything.

#### modules

Contains kernel modules to load.

### scripts

Contains scripts. Hooks will run as root on workstations if they are
found. If not, no error occurs.

| name             | description                                         |
| ---------------- | --------------------------------------------------- |
| config.sh        | contains config for the lvatest, e.g. name          |
| rollout-pre.sh   | hook that runs at rollout before testenv install    |
| rollout-post.sh  | hook that runs at rollout after testenv install     |
| boottime-pre.sh  | hook that runs at every boot before priv is fetched |
| boottime-post.sh | hook that runs at every boot after priv is fetched  |
| lightdm-session-setup.sh   | hook that runs before user is logged in   |
| lightdm-session-cleanup.sh | hook that runs after user is logged out   |
| lightdm-session-start-hook.sh | hook that runs as user before xsession |

Note: The lightdm scripts only run if lightdm is active. They will run
as root and you the username of the user being logged in or out can be
found in `$USER`. The cleanup script can be called with empty `$USER`
which means it is run after the greeter (login screen) exits. This may
happen at times where no user is beeing logged in, e.g. at reboot. So
check if it is set or use `${USER-}`

Note: Since lightdm-session-start-hook.sh is run in context of the
logged in user it does not log to the default location. Instead stdout
and stderr of it will be redirected into
`~/.lvatest.session-hook.log`.

Warning: If any command in lightdm-session-start-hook.sh fails you
will only note by looking in the log file specified above. But you can
apply some sanity checks yourself inside the hook and e.g. create a
desktop icon that points out that something went wrong. Background:
Since the hook is sourced there is no exit code that the
xsession-wrapper can check.

#### config.sh options

Format is bash code, lines with: VARNAME=value

LVATEST_NAME=test-name
LVATEST_USERNAME_PREFIX=my
LVATEST_SIMPLE_USERS_PWHASH='$6$illegal-hash'

##### How to create a password hash

You can try the following python snippet, but please do not use
`myCoolPass` as password:
```
python -c "import crypt; print(crypt.crypt('myCoolPass'))"
```

#### skel or skel.tar.gz

If a dir called "skel" exists it will be used as homedir skeleton
(blueprint) for all participant accounts. Try not to bloat up that
dir, as it will be duplicated for each user. Better symlink to
somewhere outside the users home dir if you want to place big data
into it.

If you create a file called "skel.tar.gz" it will be extracted into
the config dir. It must contain a dir called "skel". Any data
existing in "skel" dir mentioned above will be overwritten.

### sshkeys

Contains public ssh keys for all test staff users. File name must be
<username>.pub

### users

Contains info about users to be created in testenv.

#### admin-users.csv

This file contains a list of admin users for the courses technical
staff. Its syntax is:
`<username>;<Firstname SURNAME>;<pw-hash-like-in-etc-passwd>`

All admin users are member of group lvatestpriv. They have right to
run sudo. E.g. you can always "sudo bash" to get a root shell. If you
deploy an SSH Key (see above), admin users will be able to log in via
apps0 without using a password.

For infos about how to create a password hash see the config section above.

#### participants-simple.*

Files starting with "participants-simple." contain one id per line and
will be used with default password. This id will be prefixed with
`$LVATEST_USERNAME_PREFIX` from config.sh. As password hash
`$LVA_SIMPLE_USERS_PWHASH` from the same file will be used.

For infos about how to create a password hash see the config section above.

#### participants-detailed.*

Files starting with "participants-detailed." contain users in CSV
style, one user per line. The format is `username;gecos;pwhash` and
may contain another column ";grouplist" where grouplist is a comma
separated list.

For infos about how to create a password hash see the config section above.

### usb-whitelist

If this directory exists and contains files ending with `.rules` USB
access will be restricted and only specified devices are allowed. The
files format for rules is:

```
# comment

<ID_VENDOR_ID>:<ID_MODEL_ID> optional comment
```

Comments and empty lines are ignored. An example can be found on apps0
in the default database provided by ZKK for your convenience.

The ZKK will keep this file updated with default equipment on all
client hosts running the lvatest-env:
`/usr/share/zkk-lvatest-client-scripts/zkklabs-usb-base-whitelist.rules`
(the link target will show as non existing on apps0, that is not an
error). Devices consist mainly of internal USB hubs, mice and
keyboards on all apps0 hosts. Feel free to symlink it to your
configuration directory, to get automatic updates of its contents.

USB white listing is not activated by default. Symlink the example file
to activate it. Changes to rules will be applied on each bootup.

#### usb-whitelist logging

You can view a log in `/var/log/usb-whitelist-hook.log` (on the
workstations, not on apps0). A full history of all USB devices is
shown. When debugging you can change `DEBUG` to 1 in
`/sbin/lvatest-udev-whitelist-hook`. Usually you do not have to do
that.

<!--
LocalWords: lvatest testenv linux rollout config ZKK HOWTO env priv
LocalWords: img netsec gz sshkeys dir testname labname tuwien
LocalWords: rsync symlink hostnames bootime apps0 grouplist CSV
LocalWords: lvastaff username skel LVATEST USERNAME subdir ang PRIV
LocalWords: csv homedir VARNAME PWHASH hash boottime pre iptables
LocalWords: pwhash gecos lightdm png ROLLOUT illegal-hash rerollout
LocalWords: lvatestpriv usb-whitelist bootup USB xsession
 -->
